#include <stdlib.h>
#include <stdarg.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
//#include <avr/signal.h>
#include <avr/interrupt.h>
#include "GLED.h"
#include "alpha.cpp"

#define set_clk PORTD |= 0x08
#define clr_clk PORTD &= ~0x08
#define en_data DDRB |= 0x01
#define set_data PORTB |= 0x01
#define clr_data PORTB &= ~0x01
#define set_reset PORTD |= 0x04
#define clr_reset PORTD &= ~0x04


// Pinout of GLED Display
//  data blank +5VDC
// reset clock GND

unsigned long disp[5] = { 0x80000000, 0x80000000, 0x80000000, 0x80000000, 0x80000000 };
unsigned char brght = 0xff;


// - ' '

void set_alpha(char *txt)
// sets the alpha seven characters to the first seven characters in the string 'txt'
{
	int i;
	for(i = 0; i < 7; i++) {
		if(i < 5) {
			disp[i] &= ~0x7ffe0000;
			disp[i] |= (unsigned long)pgm_read_word(&alpha[(*(txt + i)) - ' ']) << 17;
		} else {
			disp[i - 2] &= ~0x0001fff8;
			disp[i - 2] |= (unsigned long)pgm_read_word(&alpha[(*(txt + i)) - ' ']) << 3;
		}
	}
}

void set_numeric(char *txt)
// Sets the numeral four characters to the first four characters in the string 'txt'
{
	int i;
	for(i = 0; i < 4; i++) {
		if(i < 2) {
			disp[i + 1] &= ~0x0001fc00;
			disp[i + 1] |= (unsigned long)pgm_read_byte(&numeral[(*(txt + i)) - ' ']) << 10;
		} else {
			disp[i - 1] &= ~0x000003f8;
			disp[i - 1] |= (unsigned long)pgm_read_byte(&numeral[(*(txt + i)) - ' ']) << 3;
			
		}
	}
}

void set_alpha_colon(unsigned char in)
// sets the alpha colon to Least-Significant 2-bits of 'in'
{
	disp[0] &= ~0x00000030;
	disp[0] |= in << 4;
}

void set_num_colon(unsigned char in)
// sets the numeral colon to Least-Significant 2-bits of 'in'
{
	disp[0] &= ~0x00018000;
	disp[0] |= (unsigned long)in << 15;
}

void set_icon(unsigned char in)
// Sets the icon byte to 'in'
{
	disp[0] &= ~0x00003fc0;
	disp[0] |= (unsigned short)in << 6;
}

void set_brightness(unsigned char in)
// Sets the icon byte to 'in'
{
	unsigned char lookup[8] = { 0x01, 0x11, 0x15, 0x55, 0x57, 0x77, 0x7f, 0xff };
	brght = lookup[in];
}


void delay(void)
{
	int i;
	for(i = 0; i < 20; i++);
}

void set_number(unsigned short n)
// Sets the numeral four characters to the right adjusted decimal of 'n'
{
	int i;
	unsigned char code;
	for(i = 3; i >= 0; i--) {
		code = (n % 10) + '0' - ' ';
		if((n == 0) && (i < 3)) code = ' ' - ' ';
		code = pgm_read_byte(&numeral[code]);
		if(i < 2) {
			disp[i + 1] &= ~0x0001fc00;
			disp[i + 1] |= (unsigned long)code << 10;
		} else {
			disp[i - 1] &= ~0x000003f8;
			disp[i - 1] |= (unsigned long)code << 3;
			
		}
		n = n / 10;
	}
}

void GLED_Clear(void)
{
	short n;
	for(n = 0; n < 5; n++) disp[n] = 0x80000000;
	GLED_Display();
}

void GLED_Display(void)
// Strobes the display once, must strobe five times to fill all elements
{
	int cnt;
	static int set = 0;
	static unsigned char bmask = 1;
	unsigned long mask = 0x80000000;
	TIFR2 = 0x1;
	en_data;
	clr_reset;
	for(cnt = 0; cnt < 30; cnt++) {
		if(disp[set] & mask) set_data;
		else clr_data;
		mask = mask >> 1;
		delay();
		set_clk;
		delay();
		clr_clk;
	}
	for(cnt = 0; cnt < 7; cnt++) {
		if(cnt == set) set_data;
		else clr_data;
		delay();
		set_clk;
		delay();
		clr_clk;
	}
	if(!(bmask & brght)) set_reset;
	bmask = bmask << 1;
	if(!bmask) bmask = 1;
	set++;
	if(set >= 5) set = 0;
}

