#include "config.h"


// Board A Configuration
unsigned char Emitter_Configurations[5][3];
// Board B Configuration


int total;  // total number to encode

void Fetch_Configuration(void)
{
	short n;
	for(n = 0; n < 15; n++) *(Emitter_Configurations[0]+n) = (unsigned char)(EEPROM_read(n + 10));
	
}

void Set_Emitter(unsigned char num)
{
	if(num == 0) {
		set_Cap(0);
		Stop_Emitter;
		TCNT1 = 0;
		PRR |= 0x08;
	} else {
		num += tb_offset;
		while(TCNT1 > (ICR1 >> 1));
		PRR &= ~0x08;
		ICR1 =  Emitter_Configurations[(short)(num)][0];
		OCR1A = Emitter_Configurations[(short)(num)][1];
		OCR1B = ICR1 - OCR1A;
		set_Cap(Emitter_Configurations[(short)(num)][2]);
		Start_Emitter;
	}
}

void Run_Emitter(short sec)
{
	char n = 1;
	short m;
	char pulse = (sec & 0x4000)?1:0;
	char sem = 1;
	sec = (sec & 0x3fff) * 32;
	PRR &= ~0x20;	// enable timer0
	// Start Timer0 Utilizing a 4x slower timing environment
	TCCR0B = 0x05;	// clk / 1024
	OSCCAL = EEPROM_read(28);	// calibration correction to 8MHz
	while(1) {
		set_LEDs(n);
		Set_Emitter(n);
		m = 1;
		while(m != sec) {
			sleep_enable();
			sleep_mode();
			sleep_disable();
			en_keys;
			if(sec) m++;
			if(pulse) Set_Emitter(0);
			if(!B2) sem = 0;
			else if(!sem) { m = sec; sem = 1; }
			en_LEDs;
		}
		n++;
		if(n > 4) n = 1;
	}
}

unsigned char gcount(unsigned char x)	
// returns the number of ones in variable 'x'
{
	unsigned char c=0; 
	while(x) {
		++c; 
		x &= x-1;
	} 
	return c;
}

void BuildPacket(char init)
{
	if(init) {
		TXpacket[2] = 0;
	} else {
		TXpacket[2]++;
	}
	TXpacket[3] = 0xf5;
}

void Run_Transmitter(void)
{
	//char txt[5] = "Sent\n";
	// utilizes a 1/8th slower environment
	//BuildPacket(1);
	while(1) {
		set_LEDs(0x1);
		TX_Packet(4);
		//PORTD |= 0x18;	// end packet
		en_keys;
		PRR = 0xef;	// turn off all hardware but WDT
		enable_watchdog(7);
		sleep_enable();
		sleep_mode();
		sleep_disable();
		//GLED_Display();
		disable_watchdog();
	}
}

/***********TX Interface*************/

void TX_Packet(unsigned char symbols) 
{
	unsigned char output, duration;
	unsigned char send;
	unsigned char n;
	PRR &= ~0x20;	// enable timer0
	// Start Timer0 Utilizing a 4x slower timing environment
	TCCR0B = 0x05;
	send = 0;
	//PORTD |= 0x08;	// stop clock
	sleep_enable();
	sleep_mode();
	sleep_disable();
	//PORTD &= ~0x18;	// start packet
	for(n = 0; n < symbols; n++) {
		if((n & 0x3) == 0) send = TXpacket[n >> 2];
		output = send >> 6;
		DDRC |= 0x18;
		PORTC = (PORTC & 0xe7) | (output << 3);
		#if wired
			// nothing
		#else
		//PORTC ^= 0x20;
		Set_Emitter(output + 1);
		#endif
		for(duration = 0; duration < 2; duration++) {
			//GLED_Display();
			//PORTD |= 0x08;	// stop clock
			sleep_enable();
			sleep_mode();
			sleep_disable();
			//PORTD &= ~0x18;	// start packet
		}
		send = send << 2;
	}
	PORTC |= 0x18;
	DDRC &= ~0x18;
	Set_Emitter(0);		// Turn off Emitter
	TCCR0B = 0x00;	// stop timer0
	PRR |= 0x20;	// disable timer0
}





