#include <avr/io.h>
#include "UART.h"

void USART_Number(unsigned long num, unsigned char base, unsigned char is_sign)
{
	unsigned char buffer[10];
	unsigned char temp;
	unsigned char n;
	if(is_sign && ((long)(num) < 0)) {
		num = -((long)(num));
		USART_Write('-');
	}
	n = 10;
	while(num > 0) {
		n--;
		temp = (unsigned char)(num % base);
		if(temp < 10) buffer[n] = (unsigned char)('0' + temp);
		else buffer[n] = (unsigned char)((temp - 10) + 'A');
		num = num / base;
	}
	if(n == 10) {
		USART_Write('0');
	}
	while(n < 10) {
		USART_Write(buffer[n]);
		n++;
	}
}

void USART_Hex(unsigned long num) 
{
	short i = 8;
	char txt[9];
	txt[8] = '\0';
	
	do {
		i--;
		if((num & 0xf) >= 10) txt[i] = (num & 0xf) - 10 + 'A';
		else txt[i] = (num & 0xf) + '0';
		num = num >> 4;
	} while(num);
	txt[--i] = ' ';
	while(i < 9) {
		/* Wait for empty transmit buffer */
		while ( !( UCSR0A & (1<<UDRE0)) )	;
		/* Put data into buffer, sends the data */
		UDR0 = txt[i];
		i++;
	}
}

void USART_Init(unsigned short baud)
// 
{
	DDRD &= 0xfe;
	DDRD |= 0x02;
	baud = 51; //(fosc / (16 * baud)) - 1;	8Mhz 9600baud
	//Set baud rate 
	UBRR0H = (unsigned char)(baud>>8);
	UBRR0L = (unsigned char)(baud);
	// Enable receiver and transmitter 
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	//Set frame format: 8data, 2stop bit 
	UCSR0C = (1<<USBS0)|(3<<UCSZ00);
}

void USART_String(char* txt)
{
	while((*txt) != '\0') {
		while(!(UCSR0A & (1<<UDRE0)));
		UDR0 = (*txt);
		if((*txt) == '\n') (*txt) = '\r';
		else txt++;
	}
}

void USART_Write(char c)
{
	//Wait for empty transmit buffer 
	while ( !( UCSR0A & (1<<UDRE0)) )	;
	//Put data into buffer, sends the data 
	UDR0 = c;
}

void EEPROM_write(unsigned int uiAddress, unsigned char ucData)
{
//Wait for completion of previous write 
while(EECR & (1<<EEPE))
;
//Set up address and Data Registers 
EEAR = uiAddress;
EEDR = ucData;
//Write logical one to EEMPE 
EECR |= (1<<EEMPE);
//Start eeprom write by setting EEPE 
EECR |= (1<<EEPE);
}

unsigned char EEPROM_read(unsigned int uiAddress)
{
//Wait for completion of previous write 
while(EECR & (1<<EEPE))
;
//Set up address register 
EEAR = uiAddress;
//Start eeprom read by writing EERE 
EECR |= (1<<EERE);
//Return data from Data Register 
return EEDR;
}

