
#include "config.h"
char menutitles[] PROGMEM = "START  SYS    TX     RX     ICR1   OCR1A  FREQ   TONE   RX TESTBENCH  SYSSUB ";
unsigned char buttons = 0;
unsigned short repeat = 80;
unsigned char tb_offset = 0;

unsigned char TXpacket[packet_size]; //= {0x01, 0x15, 0x6a, 0xbf};//, 0xa5, 0xaf};
unsigned char RXpacket[packet_size];
unsigned char magic = 0;

EMPTY_INTERRUPT(TIMER0_COMPA_vect);
EMPTY_INTERRUPT(WDT_vect);
extern unsigned char DATA_LENGTH;
unsigned char PickDest=0;

void disable_watchdog(void)
{
	// Disable watchdog timer
	MCUSR &= ~(1<<WDRF);	// Clear WDRF in MCUSR
	WDTCSR |= (1<<WDCE) | (1<<WDE);	//Write logical one to WDCE and WDE 
	WDTCSR = 0x00;	//Turn off WDT 
	set_sleep_mode(SLEEP_MODE_IDLE);
}

void enable_watchdog(char delay)
// 'delay' 0=16ms, 4=0.25s, 5=0.5s, 6=1.0s, 7=2.0s, 32=4.0s, 33=8.0s
{
	/* Start timed equence */
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	/* Set new prescaler(time-out) value = 64K cycles (~0.5 s) */
	WDTCSR = 0x40 + delay;	// 0x46 enable watchdog
	sei();
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
}


void init(void)
{
	unsigned short d;
	char text[8];
	PORTB = 0x00;
	PORTC = 0x30;
	PORTD = 0x12;
	DDRB = 0xc7;
	DDRC = 0xf9;
	DDRD = 0xfe;
	disable_watchdog();
	USART_Init(9600);
	// Configure Timer0
	OCR0A = 250;
	TCCR0A = 0x02;
	TCCR0B = 0x00;	// timer0 configured but halted
	TIMSK0 = 0x02;		// interrupt on OCR0A compare match
	set_LEDs(EEPROM_read(7));
	OCR0A = EEPROM_read(9);
	OSCCAL = EEPROM_read(28);	// calibration correction to 8MHz

	en_LEDs;
	for(d = 0; d < 8; d++) text[d] = EEPROM_read(d);
	set_alpha(text);
	//for(d = 0; d < 20000; d++) GLED_Display();
	//GLED_Clear();
	Fetch_Configuration();
	Fetch_RxConfiguration();
	// Configure power saving features
	set_sleep_mode(SLEEP_MODE_IDLE);
	//PRR = 0xed;	// 11101101	shutdown all hardware except USART
	set_brightness(EEPROM_read(8));
	sei();
	Set_Emitter(0);		// shutdown Emitter*/
}


char menu(void) 
{
	char txt[8];
	char menu = 0;
	char run = 0;
	char sem = 0;
	char temp = 0;
//	char temp2 = 0;
	char ch;
	short Vsup = 0;
//	unsigned char* var = &menu;
	unsigned char cur_ADC = 14;
	PRR &= ~0x01;
	// Enable ADC
	ADMUX = cur_ADC | 0x40;
	ADCSRA = 0Xd7;	// 11000111	single trigger
	en_keys;
	while(!run) {
		if(((sem == 0) && Bx) || (!Bx && sem)) {
			switch(menu) {
			case 0:		// Startup
				if(B1) menu = 1;
				if(B2) menu = 2;
				if(B3) menu = 3;
				break;
			case 1:		// System Menu 
				temp = EEPROM_read(8);
				if(B1 && temp) temp--;
				if(B2 && (temp < 7)) temp++;
			//	if(B3) menu = 10;
				set_brightness(temp);
				set_number(temp);
				EEPROM_write(8, temp);
				break;
			case 2:
				if(B1) menu = 6;
				if(B2) menu = 7;
				if(B3) run = 15;
				break;
			case 3:		// Receiver Sub-Menu
				if(B1) menu = 8;
				if(B2) run = 21;
				break;
			case 8:		// Receiver Testing Menu
				if(B1) run = 20;
				if(B3) menu = 9;
				break;
			case 9:		// MAC destination Menu
				if(B1) PickDest=0;
				if(B2) PickDest=1;
				if(B3) menu=11;
				break;
			case 11:		// MAC Testing Menu
				if(B1) run = 29;
				if(B2) run = 27;
				if(B3) run = 28;
				break;
			case 7:		// Tone Menu
				if(B1) run = 12;
				if(B2) run = 13;
				if(B3) run = 14;
				break;
			}
			if(!Bx) sem = 0;
			else {
				sem = 1;
				if(menu) {
					if(menu > 1) {
						for(ch = 0; ch < 8; ch++) txt[(unsigned char)ch] = pgm_read_byte(&menutitles[ch + menu * 7]);
						set_alpha_colon(0);
					} else {
						sprintf(txt, " %3d   ", Vsup);
						set_alpha_colon(1);
					}
					txt[7] = '\0';
					USART_String(txt);
					set_alpha(txt);
				}
			}
		}
		
		set_LEDs(menu);
		GLED_Display();
		if(ADCSRA & 0x10) {
			switch(cur_ADC) {
			//case 1:		// Temperature
				//Temperature = (ADC * 20) >> 6;
				//break;
			//case 2:		// Emitter voltage
			//	Vemit = ADC;
			//	sprintf(txt, " %3d %d  ", Vbat, Temperature);
			//	set_alpha_colon(1);
			//	set_alpha(txt);
			//	break;
			case 14:		// supply voltage
				Vsup = (56320 /  (ADC >> 1));
				break;
			};
			//cur_ADC++;
			if(cur_ADC > 0xe) cur_ADC = 0;
			if(cur_ADC >= 5) cur_ADC = 0xe;
			//ADMUX = cur_ADC | 0x40;
			ADCSRA = 0xd7; // 11000111 start single conversion
		}
		en_keys;
	}
	ADCSRA = 0x00;	// stop ADC
	GLED_Clear();
	PRR |= 0x09;	// shutdown ADC and Timer1
	return run;
}

void Display(){
unsigned char i;
unsigned char number;
	for(i=0;i<12*2;i++){
		number=EEPROM_read(i+70);
		USART_Number(number,10,0);
		USART_String("  ");
	
	}
};
int main(void)
{
	char run;
	init();
	//test(-338);
	//EEPROM_setup();
	//Cap_test();
	run = menu();
	switch(run) {

		case 12: Run_Emitter(2); break;
		case 13: Run_Emitter(0xa); break;
		case 14: Run_Emitter(0); break;
		case 15: Run_Transmitter(); break;
		case 20: coef_Scan();
		case 21: Receiver_Test(); break;
		case 27: Aloha();break;
		case 28: MACA(); break;
		case 29: Display(); break;
		
	}

	en_keys;
	
	return 0;
}
